
import 'dart:ui';

import 'package:wiialert/ActionSheet/WIIActionRowWidget.dart';
import 'package:wiialert/WIIAlertAction.dart';
import 'package:wiialert/ActionSheet/WIICancelActionWidget.dart';
import 'package:wiialert/ActionSheet/WIISheetContentRowWidget.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class WIIActionSheetWidge extends StatefulWidget {
  String _title;
  String _message;
  List<WIIAlertAction> _actions = new List();
  WIIAlertAction _cancelAction;

  WIIActionSheetWidge(this._title, this._message);

  void addAction(WIIAlertAction action) {
    if (action == null || action.style == null || action.title == null) {
      return;
    }
    switch (action.style) {
      case WIIAlertActionStyle.defult:
        _actions.add(action);
        break;
      case WIIAlertActionStyle.cancel:
        _cancelAction = action;
        break;
      case WIIAlertActionStyle.destructive:
        _actions.add(action);
    }
  }

  @override
  State<StatefulWidget> createState() {
    return WIIActionSheetWidgeState();
  }

  void showIn(BuildContext context) {
    showModalBottomSheet(
      context: context,
      backgroundColor: Colors.transparent,
      builder: (BuildContext context) {
        return this;
      },
    );
  }
}

class WIIActionSheetWidgeState extends State<WIIActionSheetWidge> {

  double _bottom = 0;
  double _width = 0;

  double _contentHeight = 0;
  double _msgHeight = 0;
  double _actionHeight = 0;
  double _cancelHeight = 0;

  @override
  Widget build(BuildContext context) {
    _width = MediaQuery.of(context).size.width;
    _bottom = MediaQuery.of(context).padding.bottom;
    calculateHeight();

    return new Container(
      height: _contentHeight,
      child: new Stack(
        children: <Widget>[
          createContentPositioned(),
          createCancelPositioned()
        ],
      ),
    );
  }

  Widget createContentPositioned() {
    return new Positioned(
      left: 8,
      right: 8,
      top: 0,
      height: _actionHeight + _msgHeight,
      child: new ClipRRect(
        borderRadius: BorderRadius.circular(10),
        child: new Container(
          color: Color(0xFFFFFFFF),
          child: new Column(
            children: createListRows(),
          ),
        ),
      ),
    );
  }

  Widget createCancelPositioned() {
    return new Positioned(
      left: 8,
      right: 8,
      top: _actionHeight + _msgHeight,
      height: _cancelHeight,
      child: new Container(
        padding: EdgeInsets.only(top: 8),
        child: new ClipRRect(
          borderRadius: BorderRadius.circular(10),
          child: createCancelAction(),
        ),
      ),
    );
  }

  bool hasCancelAction() {
    return widget._cancelAction != null;
  }

  List<Widget> createListRows() {
    List<Widget> rows = new List<Widget>();

    var contentRow = WIISheetContentRowWidget(widget._title, widget._message, _msgHeight);
    rows.add(contentRow);

    var defalutRows = createActionRows();
    rows.addAll(defalutRows);

    return rows;
  }

  List<Widget> createActionRows() {
    List<Widget> rows = new List<Widget>();

    for (WIIAlertAction action in widget._actions) {
      var row = WIIActionRowWidget(action, false, 57, () {
        tapAction();
      });
      rows.add(row);
    }
    return rows;
  }

  Widget createCancelAction() {
    return WIICancelActionWidget(widget._cancelAction, () {
      tapAction();
    });
  }

  void tapAction() {
    Navigator.of(context).pop();
  }


  void calculateHeight() {
    calculateHeadHeight();
    calculateActionHeight();
    calculateCancelHeight();
    double bottom = _bottom + 8;
    _contentHeight = _msgHeight + _actionHeight + _cancelHeight + bottom;
  }

  void calculateHeadHeight() {
    double top = 14;
    if (widget._title != null && widget._title.length > 0) {
      double height = calculateTextHeight(widget._title);
      top += (height + 14);
    }
    if (widget._message != null && widget._message.length > 0) {
      double height = calculateTextHeight(widget._message);
      top += (height + 14);
    }
    if (top == 14) {
      _msgHeight = 0;
    } else {
      _msgHeight = top;
    }
  }

  double calculateTextHeight(String text) {
    TextStyle style = new TextStyle(fontSize: 13, fontWeight: FontWeight.bold);
    TextSpan span = TextSpan(text: text, style: style);
    TextPainter painter = TextPainter(text: span, textDirection: TextDirection.ltr);
    double maxWidth = _width - 52;
    painter.layout(maxWidth: maxWidth);
    return painter.size.height;
  }

  void calculateActionHeight() {
    _actionHeight = widget._actions.length * 57.0;
  }

  void calculateCancelHeight() {
    if (widget._cancelAction != null) {
      _cancelHeight = 57.0 + 8.0;
    } else {
      _cancelHeight = 0;
    }
  }
}