
import 'package:wiialert/WIIAlertAction.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class WIIAlertHorizontalActionWidget extends StatelessWidget {
  List<WIIAlertAction> _actions;

  VoidCallback _tapAction;

  WIIAlertHorizontalActionWidget(this._actions, this._tapAction);

  @override
  Widget build(BuildContext context) {
    return new Stack(
      children: <Widget>[
        new Positioned(
          left: 0,
          right: 0,
          top: 0,
          height: 0.5,
          child: new Container(
            color: Color(0xFFCCCCCC),
          ),
        ),
        new Positioned(
          left: 0,
          right: 0,
          top: 0.5,
          bottom: 0,
          child: Flex(
            direction: Axis.horizontal,
            children: createExpands(),
          ),
        )
      ],
    );
  }

  List<Widget> createExpands() {
    List<Widget> expands = new List<Widget>();
    for (int i = 0; i < _actions.length; i ++) {
      WIIAlertAction action = _actions[i];
      Widget widget = new Expanded(
        flex: 1,
        child: new Stack(
          children: createExpandChilds(action, (i == 0)),
        ),
      );
      expands.add(widget);
    }
    return expands;
  }

  List<Widget> createExpandChilds(WIIAlertAction action, bool hideLine) {
    List<Widget> childs = new List<Widget>();
    if (hideLine == false) {
      Widget line = createLine();
      childs.add(line);
    }

    Widget title = createTitle(action);
    childs.add(title);
    return childs;
  }

  Widget createTitle(WIIAlertAction action) {
    return new Positioned(
      left: 1,
      right: 0,
      top: 0,
      bottom: 0,
      child: new GestureDetector(
        onTap: () {
          respondsToGesture(action);
        },
        child: new Container(
          alignment: Alignment.center,
          child: new Text(
            action.title,
            style: textStyle(action),
          ),
        ),
      ),
    );
  }

  Widget createLine() {
    return new Positioned(
      left: 0,
      width: 0.5,
      top: 0,
      bottom: 0,
      child: new Container(
        color: Color(0xFFCCCCCC),
      ),
    );
  }

  TextStyle textStyle(WIIAlertAction action) {
    if (action.style == WIIAlertActionStyle.destructive) {
      return new TextStyle(
        fontSize: 17,
        color: Colors.red
      );
    } else if (action.style == WIIAlertActionStyle.cancel) {
      return new TextStyle(
        fontSize: 17,
        fontWeight: FontWeight.bold,
        color: Colors.blue
      );
    } else {
      return new TextStyle(
          fontSize: 17,
          color: Colors.blue
      );
    }
  }

  void respondsToGesture(WIIAlertAction action) {
    if (action.handler != null) {
      action.handler(action);
    }
    if (_tapAction != null) {
      _tapAction();
    }
  }
}

