
import 'package:wiialert/WIIAlertAction.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class WIIAlertVerticalActionWidget extends StatelessWidget {
  List<WIIAlertAction> _actions;

  VoidCallback _tapAction;

  WIIAlertVerticalActionWidget(this._actions, this._tapAction);

  @override
  Widget build(BuildContext context) {
    return new Flex(
      direction: Axis.vertical,
      children: createExpands(),
    );
  }

  List<Widget> createExpands() {
    List<Widget> expands = new List<Widget>();
    for(WIIAlertAction action in _actions) {
      Widget child = createExpandChild(action);
      expands.add(child);
    }
    return expands;
  }

  Widget createExpandChild(WIIAlertAction action) {
    return new Expanded(
      flex: 1,
      child: new Stack(
        children: <Widget>[
          createLine(),
          createContent(action)
        ],
      ),
    );
  }

  Widget createLine() {
    return new Positioned(
      left: 0,
      right: 0,
      top: 0,
      height: 0.5,
      child: new Container(
        color: Color(0xFFCCCCCC),
      ),
    ) ;
  }

  Widget createContent(WIIAlertAction action) {
    return new Positioned(
      left: 0,
      right: 0,
      top: 0.5,
      bottom: 0,
      child: new GestureDetector(
        onTap: () {
          respondsToGesture(action);
        },
        child: new Container(
          alignment: Alignment.center,
          child: new Text(
            action.title,
            style: action.textStyle(17),
          ),
        ),
      ),
    );
  }


  void respondsToGesture(WIIAlertAction action) {
    if (action.handler != null) {
      action.handler(action);
    }
    if (_tapAction != null) {
      _tapAction();
    }
  }
}