
import 'package:wiialert/Alert/WIIAlertContentWidget.dart';
import 'package:wiialert/Alert/WIIAlertHorizontalActionWidget.dart';
import 'package:wiialert/Alert/WIIAlertVerticalActionWidget.dart';
import 'package:wiialert/WIIAlertAction.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class WIIAlertWidget extends StatefulWidget {
  String _title;
  String _message;
  List<WIIAlertAction> _actions = new List();
  WIIAlertAction _cancelAction;

  WIIAlertWidget(this._title, this._message);

  void addAction(WIIAlertAction action) {
    if (action == null || action.style == null || action.title == null) {
      return;
    }
    switch (action.style) {
      case WIIAlertActionStyle.defult:
        _actions.add(action);
        break;
      case WIIAlertActionStyle.cancel:
        _cancelAction = action;
        break;
      case WIIAlertActionStyle.destructive:
        _actions.add(action);
    }
  }

  @override
  State<StatefulWidget> createState() {
    return WIIAlertWidgetState();
  }

  void showIn(BuildContext context) {
    Navigator.push(
      context,
      PageRouteBuilder(
        opaque: false,
        transitionDuration: Duration(milliseconds: 250), //动画时间为500毫秒
        pageBuilder: (BuildContext context, Animation animation,
            Animation secondaryAnimation) {
          return new FadeTransition(
            //使用渐隐渐入过渡,
            opacity: animation,
            child: this, //路由B
          );
        },
      ),
    );
  }
}

class WIIAlertWidgetState extends State<WIIAlertWidget> {

  double _width = 0;

  double _contentHeight = 0;
  double _msgHeight = 0;
  double _actionHeight = 0;

  @override
  Widget build(BuildContext context) {
    _width = MediaQuery.of(context).size.width;
    calculateHeight();
    return new Scaffold(
      backgroundColor: Colors.black54,
      body: new Container(
        padding: EdgeInsets.only(left: 53, right: 53),
        color: Color(0x00000000),
        alignment: Alignment.center,
        child: ClipRRect(
          borderRadius: BorderRadius.circular(10),
          child: new Container(
            height: _contentHeight,
            color: Colors.white,
            child: new Stack(
              children: <Widget>[
                createHeadContentPositioned(),
                createActionPositioned()
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget createHeadContentPositioned() {
    return new Positioned(
      left: 0,
      right: 0,
      top: 0,
      height: _msgHeight,
      child: WIIAlertContentWidget(widget._title, widget._message),
    );
  }

  Widget createActionPositioned() {
    List<WIIAlertAction> actions = allActions();
    Widget child = null;
    int count = actionsCount();
    if (count <= 2) {
      child = WIIAlertHorizontalActionWidget(actions, () {
        tapAction();
      });
    } else {
      child = WIIAlertVerticalActionWidget(actions, () {
        tapAction();
      });
    }

    return new Positioned(
      left: 0,
      right: 0,
      top: _msgHeight,
      bottom: 0,
      child: child,
    );
  }

  void tapAction() {
    Navigator.of(context).pop();
  }

  void calculateHeight() {
    calculateHeadHeight();
    calculateActionHeight();
    _contentHeight = _msgHeight + _actionHeight;
  }

  void calculateHeadHeight() {
    double top = 21;
    if (widget._title != null && widget._title.length > 0) {
      double height = calculateTextHeight(widget._title, 17);
      top += (height + 5);
    }
    if (widget._message != null && widget._message.length > 0) {
      double height = calculateTextHeight(widget._message, 13);
      top += (height + 5);
    }
    if (top == 21) {
      _msgHeight = 0;
    } else {
      _msgHeight = top + 16;
    }
  }

  double calculateTextHeight(String text, double fontSize) {
    TextStyle style = new TextStyle(fontSize: fontSize, fontWeight: FontWeight.bold);
    TextSpan span = TextSpan(text: text, style: style);
    TextPainter painter = TextPainter(text: span, textDirection: TextDirection.ltr);
    double maxWidth = _width - 148;
    painter.layout(maxWidth: maxWidth);
    return painter.size.height;
  }

  void calculateActionHeight() {
    int count = actionsCount();
    if (count <= 2) {
      _actionHeight = 45;
    } else {
      _actionHeight = 45.0 * count;
    }
  }

  int actionsCount() {
    if (widget._cancelAction == null) {
      return widget._actions.length;
    } else {
      return widget._actions.length + 1;
    }
  }

  List<WIIAlertAction> allActions() {
    List<WIIAlertAction> actions = new List<WIIAlertAction>();
    actions.addAll(widget._actions);
    if (widget._cancelAction != null) {
      actions.add(widget._cancelAction);
    }
    return actions;
  }
}